#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains DCC abstract UI implementation
"""

from __future__ import print_function, division, absolute_import

from artella.core.dcc import reroute
from artella.core.utils import abstract


